@@ -1,6 +1,10 @@
 package Devel::Camelcadedb;
 # must be quoted to work correctly with JSON protocol
-our $VERSION = "v2019.1"; # DO NOT REMOVE FUCKING v, IT KEEPS PROPER VERSIONING
+our $VERSION = "v2020.3"; # DO NOT REMOVE FUCKING v, IT KEEPS PROPER VERSIONING
+
+# to ensure protocol compatibility between the IDE and the debugger, we will use $API_VERSION variable, to be able
+# to bump debugger version without necessity to update IDE part.
+my $API_VERSION = "2019.1";
 
 # http://perldoc.perl.org/DB.html
 # http://perldoc.perl.org/perldebug.html
@@ -968,6 +972,10 @@ sub _get_next_command
         }
         unless ($new_line_index > -1)
         {
+            if (scalar @saved) {
+                ($@, $!, $^E, $,, $/, $\, $^W) = @saved;
+                @saved = ();
+            }
             print STDERR "Buffer $input_buffer has no newlines in it and nothing is in the socket\n";
             exit -1;
         }
@@ -1288,7 +1296,7 @@ sub _set_up_after_connect
 
     _send_data_to_debugger( +{
         event   => 'READY',
-        version => $VERSION,
+        version => $API_VERSION,
     } );
     _report "Waiting for set up data..." if $_dev_mode;
     my $set_up_data = <$_debug_socket>;
@@ -1485,7 +1493,8 @@ EOM
 
         if ($condition && !_eval_expression( $condition )->{result})
         {
-            ( $@, $!, $^E, $,, $/, $\, $^W ) = @saved;
+            ($@, $!, $^E, $,, $/, $\, $^W) = @saved;
+            @saved = ();
             $_internal_process = 0;
             return;
         }
@@ -1498,7 +1507,8 @@ EOM
         if (!$breakpoint->{suspend})
         {
             _send_breakpoint_reached_event( $breakpoint );
-            ( $@, $!, $^E, $,, $/, $\, $^W ) = @saved;
+            ($@, $!, $^E, $,, $/, $\, $^W) = @saved;
+            @saved = ();
             $_internal_process = 0;
             return;
         }
@@ -1537,7 +1547,8 @@ EOM
     _event_handler( $breakpoint ) unless $skip_event_handler;
 
     $_internal_process = 0;
-    ( $@, $!, $^E, $,, $/, $\, $^W ) = @saved;
+    ($@, $!, $^E, $,, $/, $\, $^W) = @saved;
+    @saved = ();
     ();
 }
 
@@ -1825,36 +1836,48 @@ sub is_connected
 
 sub _connect
 {
+    my $_perl5_debug_host = $ENV{PERL5_DEBUG_HOST};
+    my $_perl5_debug_port = $ENV{PERL5_DEBUG_PORT};
+
+    # ${^TAINT} will be truthy if taint mode is on.
+    if (${^TAINT})
+    {
+        # The debugger will fail with "Insecure dependency in connect..."
+        # if we do not untaint the host and port variables, so we do so here.
+        ($_perl5_debug_host) = $_perl5_debug_host =~ /(.*)/;
+        ($_perl5_debug_port) = $_perl5_debug_port =~ /(.*)/;
+    }
+
     my ($attempts, $allow_fail) = @_;
     # http://perldoc.perl.org/perlipc.html#Sockets%3a-Client%2fServer-Communication
     if ($ENV{PERL5_DEBUG_ROLE} eq 'server')
     {
-        printf STDERR "Listening for the IDE connection at %s:%s...\n", $ENV{PERL5_DEBUG_HOST}, $ENV{PERL5_DEBUG_PORT};
+        printf STDERR "Listening for the IDE connection at %s:%s...\n", $_perl5_debug_host, $_perl5_debug_port;
         my $_server_socket = IO::Socket::INET->new(
             Listen    => 1,
-            LocalAddr => $ENV{PERL5_DEBUG_HOST},
-            LocalPort => $ENV{PERL5_DEBUG_PORT},
+            LocalAddr => $_perl5_debug_host,
+            LocalPort => $_perl5_debug_port,
             ReuseAddr => 1,
             Proto     => 'tcp',
-        ) || die "Error binding to $ENV{PERL5_DEBUG_HOST}:$ENV{PERL5_DEBUG_PORT}";
+        ) || die "Error binding to ${_perl5_debug_host}:${_perl5_debug_port}";
         $_debug_packed_address = accept( $_debug_socket, $_server_socket );
     }
     else
     {
         foreach my $attempt (1 .. $attempts)
         {
-            printf STDERR "($attempt)Connecting to the IDE from process %s at %s:%s...\n", $$, $ENV{PERL5_DEBUG_HOST},
-                $ENV{PERL5_DEBUG_PORT};
+            printf STDERR "($attempt)Connecting to the IDE from process %s at %s:%s...\n", $$, ${_perl5_debug_host},
+                ${_perl5_debug_port};
             $_debug_socket = IO::Socket::INET->new(
-                PeerAddr  => $ENV{PERL5_DEBUG_HOST},
-                PeerPort  => $ENV{PERL5_DEBUG_PORT},
+                PeerAddr  => $_perl5_debug_host,
+                PeerPort  => $_perl5_debug_port,
                 ReuseAddr => 1,
                 Proto     => 'tcp',
             );
             last if $_debug_socket || $attempt == $attempts;
             sleep( 1 ); # this is kinda hacky
         }
-        die "Error connecting to $ENV{PERL5_DEBUG_HOST}:$ENV{PERL5_DEBUG_PORT}" if !$_debug_socket && !$allow_fail;
+        die "Error connecting to ${_perl5_debug_host}:${_perl5_debug_port}" if !$_debug_socket && !$allow_fail;
     }
     _set_up_after_connect( $allow_fail ) if $_debug_socket;
 }
